/*******************************************************************************
**                           dumpable.cpp
**                             part of
**                konserve -- A small backup application
**			 -------------------
**  copyright: (C) 2002 - 2004 by Florian Simnacher
**  email    : simnacher AT gmx DOT de
*******************************************************************************/

/*******************************************************************************
**  This program is free software; you can redistribute it and/or             **
**  modify it under the terms of the GNU General Public License               **
**  as published by the Free Software Foundation; either version 2            **
**  of the License, or (at your option) any later version.                    **
**                                                                            **
**  This program is distributed in the hope that it will be useful,           **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of            **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
**  GNU General Public License for more details.                              **
**                                                                            **
**  You should have received a copy of the GNU General Public License         **
**  along with this program; if not, write to the Free Software               **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    **
**  MA  02111-1307, USA.                                                      **
*******************************************************************************/

// KDE Includes
#include <kdebug.h>

// My includes
#include "dumpable.h"

////////////////////////////////////////////////////////////////////////////////
///////                  Class Dumpable                                  ///////
////////////////////////////////////////////////////////////////////////////////

QPtrList<Dumpable> Dumpable::mDumpableList  = * new QPtrList<Dumpable>;
QPtrList<Dumpable> Dumpable::mAlreadyDumped = * new QPtrList<Dumpable>;
bool               Dumpable::mInDumpAll     = false;

Dumpable::Dumpable( QString class_name, QString instance_name )
    : mClassName( class_name ),
      mInstanceName( instance_name )
{
    kdDebug(0) << "false;Dumpable::Dumpable" << endl;
    mDumpableList.append( this );
}

Dumpable::Dumpable( const Dumpable& d )
    : mClassName( d.mClassName )
{
    kdDebug(0) << "Dumpable::Dumpable" << endl;
    mDumpableList.append( this );
}

Dumpable::~Dumpable()
{
    kdDebug(0) << "Dumpable::~Dumpable" << endl;
    if (! mDumpableList.remove( this ) )
        Q_ASSERT( false );  // Should not get reached
    if ( -1 != mAlreadyDumped.find( this ) )
        mAlreadyDumped.remove( this );
}

void Dumpable::operator=( const Dumpable& d )
{
    kdDebug(0) << "Dumpable::operator=" << endl;
    mClassName = d.mClassName;
    mDumpableList.append( this );
}

QString Dumpable::dump()
{
    kdDebug(0) << "Dumpable::dump" << endl;
    if ( mInDumpAll && ( -1 == mAlreadyDumped.find( this )))
        return QString::null;
    mAlreadyDumped.remove( this );

    QString tmp( "++++++++++ Begin: " + mClassName );
    if ( QString::null != mInstanceName )
        tmp.append(", " + mInstanceName );
    tmp.append( ": \n" + dumpInternal() );
    tmp.append( "++++++++++ End: " + mClassName + "\n" );

    return tmp;
}

QString Dumpable::dumpAll()
{
    kdDebug(0) << "Dumpable::dumpAll" << endl;
    QString tmp;
    Dumpable *d;

    mAlreadyDumped = mDumpableList;
    mInDumpAll     = true;

    for ( d = mDumpableList.first(); d; d = mDumpableList.next() )
    {
        tmp += d->dump();
    }
    mAlreadyDumped.clear();
    mInDumpAll = false;

    return tmp;
}
