/*******************************************************************************
**                        profilemanager.cpp
**                             part of
**                konserve -- A small backup application
**			 -------------------
**  copyright: (C) 2002 - 2004 by Florian Simnacher
**  email    : simnacher AT gmx DOT de
*******************************************************************************/

/*******************************************************************************
**  This program is free software; you can redistribute it and/or             **
**  modify it under the terms of the GNU General Public License               **
**  as published by the Free Software Foundation; either version 2            **
**  of the License, or (at your option) any later version.                    **
**                                                                            **
**  This program is distributed in the hope that it will be useful,           **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of            **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
**  GNU General Public License for more details.                              **
**                                                                            **
**  You should have received a copy of the GNU General Public License         **
**  along with this program; if not, write to the Free Software               **
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    **
**  MA  02111-1307, USA.                                                      **
*******************************************************************************/

// Qt includes
#include <qdatetime.h>

// KDE includes
#include <kmessagebox.h>
#include <klocale.h>

// My includes
#include "profilemanager.h"
#include <kdebug.h>

////////////////////////////////////////////////////////////////////////////////
///////               class ProfileManager                               ///////
////////////////////////////////////////////////////////////////////////////////

ProfileManager::ProfileManager()
    : Dumpable( "ProfileManager" )
{
    kdDebug(0) << "ProfileManager::ProfileManager" << endl;
    mProfileList.setAutoDelete( true );
}

ProfileManager::~ProfileManager()
{
    kdDebug(0) << "ProfileManager::~ProfileManager" << endl;
    // EVIL DIRTY UGLY HACK that shows that we do not have the right design
    BackupProfile *b;
    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        Subject::_evilHack_SaveBackupProfile( b );
    }
}

void ProfileManager::insertCalled( BackupProfile *b )
{
    kdDebug(0) << "ProfileManager::insertCalled" << " b=" << b << endl;
    insertProfile( b );
}

void ProfileManager::changeCalled( BackupProfile *b )
{
    kdDebug(0) << "ProfileManager::changeCalled" << " b=" << b << endl;
    changeProfile( b );
}
void ProfileManager::removeCalled( QString s )
{
    kdDebug(0) << "ProfileManager::removeCalled" << " s=" << s << endl;
    removeProfile( s );
}

bool ProfileManager::existsCalled( QString s )
{
    kdDebug(0) << "ProfileManager::existsCalled" << " s=" << s << endl;
    return 0L != getProfile( s );
}

void ProfileManager::restoreCalled( QString s )
{
    kdDebug(0) << "ProfileManager::restoreCalled" << " s=" << s << endl;
    getProfile( s )->restore();
}

QString ProfileManager::dumpInternal()
{
    kdDebug(0) << "ProfileManager::dumpInternal" << endl;
    BackupProfile *b;
    QString        tmp;

    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        tmp += b->dump();
    }

    return tmp;
}

void ProfileManager::activate()
{
    kdDebug(0) << "ProfileManager::activate" << endl;
    BackupProfile *b;
    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        b->activate();
    }
}

void ProfileManager::slotDoBackup( QString identifier )
{
    kdDebug(0) << "ProfileManager::slotDoBackup" << " identifier=" << identifier << endl;
    Q_ASSERT( existsCalled( identifier )  );

    getProfile( identifier )->slotDoOneTimeBackup();
}

void ProfileManager::insertProfile( BackupProfile *bp )
{
    kdDebug(0) << "ProfileManager::insertProfile" << " bp=" << bp << endl;
    if ( 0L != getProfile( bp->getIdentifier() ) )
        changeProfile( bp );
    else
        newProfile( bp );

    bp->activate();
    return;
}

void ProfileManager::newProfile( BackupProfile *bp )
{
    kdDebug(0) << "ProfileManager::newProfile" << " bp=" << bp << endl;
    Q_ASSERT( 0L == getProfile( bp->getIdentifier() ) );

    Q_CHECK_PTR( bp );
    mProfileList.append( bp );
}

void ProfileManager::changeProfile( BackupProfile *bp )
{
    kdDebug(0) << "ProfileManager::changeProfile" << " bp=" << bp << endl;
    Q_ASSERT( 0L != getProfile( bp->getIdentifier() ));

    int index = mProfileList.find( getProfile( bp->getIdentifier() ) );
    mProfileList.remove( index );
    mProfileList.insert( index, bp );
}

void ProfileManager::removeProfile( QString id )
{
    kdDebug(0) << "ProfileManager::removeProfile" << " id=" << id << endl;
    const BackupProfile * const b = getProfile( id );
    if ( 0L == (long int) b )
    {
        KMessageBox::error(
            0L,
            i18n( "Sorry, could not remove profile: %1.\n"
                  "The Identifier is not in the list" ).arg( id )
            );
        return;
    }

    int index = mProfileList.find( b );
    Q_ASSERT( -1 != index );

    if ( !mProfileList.remove( index ) )
        Q_ASSERT( false );
}

BackupProfile* ProfileManager::getProfile( QString id )
{
    kdDebug(0) << "ProfileManager::getProfile" << " id=" << id << endl;
    BackupProfile *b      = 0L;
    BackupProfile *result = 0L;

    for ( b = mProfileList.first(); b; b = mProfileList.next() )
    {
        if ( id == b->getIdentifier() )
        {
            if ( 0L != result )
                Q_ASSERT( false ); // Consistency check, no Identifier twice
            result = b;
        }
    }
    return result;
}

#include "profilemanager.moc"
